
// OpenGL
#define NOMINMAX
#include <windows.h>
#include <GL/gl.h>
#include <GL/glut.h>


// std libs
#include <stdio.h>
#include <math.h>

// own header file
#include "actordrawing.h"

#include "plane.h"
#include "sphere.h"
#include "trianglemesh.h"

NxMaterialIndex ActorDrawing::materialDefault;
NxMaterialIndex ActorDrawing::materialGrass;
NxMaterialIndex ActorDrawing::materialIce;
NxMaterialIndex ActorDrawing::materialMud;
NxMaterialIndex ActorDrawing::materialRock;
NxMaterialIndex ActorDrawing::materialSun;
NxMaterialIndex ActorDrawing::materialViolet;

ActorDrawing::ActorDrawing(void)
{
}
ActorDrawing::~ActorDrawing(void)
{
}
void ActorDrawing::drawPlane(NxShape *shape) {
	NxPlaneShape *planeShape = (NxPlaneShape*)shape;
	Plane & plane = *(Plane*)planeShape->userData;

	// all our planes are like mud
	setMaterial(materialMud);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, plane.texture[0]);
	glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, 0.0f, -1.0f);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, 0.0f, -1.0f);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, 0.0f,  1.0f);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, 0.0f,  1.0f);	// Top Left Of The Texture and Quad
	glEnd();
	glDisable(GL_TEXTURE_2D);
}
void ActorDrawing::drawSphere(NxShape *shape) {
	NxSphereShape *sphereShape = (NxSphereShape*)shape;
	Sphere & sphere = *(Sphere*)sphereShape->userData;

	NxReal radius = sphereShape->getRadius();

	// all our balls are like ice
	setMaterial(ActorDrawing::materialIce);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, sphere.texture[0]);
	GLUquadric *qobj = gluNewQuadric();
	gluQuadricTexture(qobj, GL_TRUE);
	gluSphere(qobj, radius, 20, 20);
	glDisable(GL_TEXTURE_2D);
}
void ActorDrawing::drawBox(NxShape *shape) {
	NxVec3 dim = ((NxBoxShape*)shape)->getDimensions();

	// all our boxes have the default material
	setMaterial(ActorDrawing::materialDefault);

	GLfloat left = -dim.x;
	GLfloat right = dim.x;
	GLfloat bottom = -dim.y;
	GLfloat top = dim.y;
	GLfloat front = dim.z;
	GLfloat back = -dim.z;

	// front and back. make sure you look at it from outside!
	glPushMatrix();
	glTranslatef(0, 0, front);
	glRectf(left, bottom, right, top);
	glTranslatef(0, 0, -2.0*dim.z);
	glRotatef(180.0, 0, 1, 0);
	glRectf(left, bottom, right, top);
	glPopMatrix();

	// left and right
	glPushMatrix();
	glRotatef(90.0, 0, 1, 0);
	glTranslatef(0, 0, left);
	glRectf(front, bottom, back, top);
	glTranslatef(0, 0, 2.0*dim.x);
	glRotatef(180.0, 0, 1, 0);
	glRectf(front, bottom, back, top);
	glPopMatrix();

	// bottom and top
	glPushMatrix();
	glRotatef(90.0, 1, 0, 0);
	glTranslatef(0, 0, bottom);
	glRectf(left, front, right, back);
	glTranslatef(0, 0, 2.0*dim.y);
	glRotatef(180.0, 0, 1, 0);
	glRectf(left, front, right, back);
	glPopMatrix();
}
void ActorDrawing::drawCapsule(NxShape *shape) {
	NxCapsuleShape *capsule = (NxCapsuleShape*)shape;
	NxReal height = capsule->getHeight();
	NxReal radius = capsule->getRadius();

	// all our caspules are like rock
	setMaterial(ActorDrawing::materialRock);

	GLuint & texture = *(GLuint*)capsule->userData;

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texture);

	glPushMatrix();
	glRotatef(-90.0, 1, 0, 0);
	// cylinder
	GLUquadric *qobj = gluNewQuadric();
	gluQuadricTexture(qobj, GL_TRUE);
	gluCylinder(qobj, radius, radius, height, 10, 10);
	// top disk
	glTranslatef(0, 0, height);
	qobj = gluNewQuadric();
	gluQuadricTexture(qobj, GL_TRUE);
	gluDisk(qobj, 0, radius, 10, 10);
	// bottom disk
	glTranslatef(0, 0, -height);
	glRotatef(180.0, 1, 0, 0);
	qobj = gluNewQuadric();
	gluQuadricTexture(qobj, GL_TRUE);
	gluDisk(qobj, 0, radius, 10, 10);
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);
}
void ActorDrawing::drawMesh(NxShape *shape) {
	NxTriangleMeshShape *meshShape = (NxTriangleMeshShape*)shape;
	TriangleMesh & mesh = *(TriangleMesh*)meshShape->userData;

	NxU32 numFaces = mesh.numFaces;
	NxU32 numVertices = mesh.numVertices;
	NxVec3* vertices = mesh.vertices;
	NxVec3* normals = mesh.normals;
	NxU32* faces = mesh.faces;
	NxMaterialIndex* materials = mesh.materials;
	GLuint* textures = mesh.textures;

	glDisable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);
	for(NxU32 i=0; i<numFaces; i++) {

		// we have a texture and a material for every face
		glBindTexture(GL_TEXTURE_2D, textures[i]);
		setMaterial(materials[i]);

		glBegin(GL_TRIANGLES);

		glTexCoord2f(0.0f, 0.0f);
		glNormal3fv(&normals[faces[i*3+0]].x);
		glVertex3fv(&vertices[faces[i*3+0]].x);

		glTexCoord2f(1.0f, 1.0f);
		glNormal3fv(&normals[faces[i*3+1]].x);
		glVertex3fv(&vertices[faces[i*3+1]].x);

		glTexCoord2f(0.0f, 1.0f);
		glNormal3fv(&normals[faces[i*3+2]].x);
		glVertex3fv(&vertices[faces[i*3+2]].x);

		glEnd();
	}
	glEnable(GL_CULL_FACE);
	glDisable(GL_TEXTURE_2D);
}

void ActorDrawing::setMaterial(NxMaterialIndex material) {

		// ICE
		// white, quite specular, small emission and very shiny
		if		  (material == ActorDrawing::materialIce) {
			float ambient[]	  = { 0.7f, 0.7f, 0.7f, 1.0f };
			float diffuse[]	  = { 0.8f, 0.8f, 0.8f, 1.0f };
			float specular[]  = { 0.5f, 0.5f, 0.5f, 1.0f };
			float emission[]  = { 0.3f, 0.3f, 0.3f, 1.0f };
			float shininess[] = { 20.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

		// ROCK
		// gray, little specular, no emission, little shiny
		} else if (material == ActorDrawing::materialRock) {
			float ambient[]	  = { 0.2f, 0.2f, 0.2f, 1.0f };
			float diffuse[]	  = { 0.1f, 0.1f, 0.1f, 1.0f };
			float specular[]  = { 0.1f, 0.1f, 0.1f, 1.0f };
			float emission[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
			float shininess[] = { 5.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

		// MUD
		// redish, not specular, no emission, not shiny
		} else if (material == ActorDrawing::materialMud) {
			float ambient[]	  = { 0.6f, 0.4f, 0.4f, 1.0f };
			float diffuse[]	  = { 1.0f, 0.6f, 0.6f, 1.0f };
			float specular[]  = { 1.0f, 0.0f, 0.0f, 1.0f };
			float emission[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
			float shininess[] = { 0.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

		// GRASS
		// green, little specular, little emission, little shiny
		} else if (material == ActorDrawing::materialGrass) {
			float ambient[]	  = { 0.1f, 0.5f, 0.1f, 1.0f };
			float diffuse[]	  = { 0.1f, 0.5f, 0.1f, 1.0f };
			float specular[]  = { 0.1f, 0.5f, 0.1f, 1.0f };
			float emission[]  = { 0.1f, 0.1f, 0.1f, 1.0f };
			float shininess[] = { 0.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

		// SUN
		// yellow, white emission
		} else if (material == ActorDrawing::materialSun) {
			float ambient[]	  = { 0.5f, 0.5f, 0.1f, 1.0f };
			float diffuse[]	  = { 0.5f, 0.5f, 0.1f, 1.0f };
			float specular[]  = { 0.5f, 0.5f, 0.1f, 1.0f };
			float emission[]  = { 0.1f, 0.1f, 0.1f, 1.0f };
			float shininess[] = { 0.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

		// VIOLET
		// violet, white emission
		} else if (material == ActorDrawing::materialViolet) {
			float ambient[]	  = { 0.4f, 0.0f, 0.5f, 1.0f };
			float diffuse[]	  = { 0.4f, 0.0f, 0.5f, 1.0f };
			float specular[]  = { 0.4f, 0.0f, 0.5f, 1.0f };
			float emission[]  = { 0.1f, 0.0f, 0.1f, 1.0f };
			float shininess[] = { 0.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

		// DEFAULT
		// grey, not specular, no emission, not shiny
		} else {
			float ambient[]	  = { 0.2f, 0.2f, 0.2f, 1.0f };
			float diffuse[]	  = { 0.4f, 0.4f, 0.4f, 1.0f };
			float specular[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
			float emission[]  = { 0.0f, 0.0f, 0.0f, 1.0f };
			float shininess[] = { 0.0f };
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
		}
}