#include "camera.h"

#include "environment.h"

Camera::Camera()
{
	angle = 0.0;
	speed = 0.005;

	// maybe we add some small constant here
	radius = Environment::radius;
	height = Environment::height;
}

Camera::Camera(double aangle, double aradius, double aspeed)
{
	int sign;
	if (abs(aangle) > 2*M_PI)
	{
		sign = (aangle > 0) ? 1 : -1;									// signum von aangle
		angle = aangle - sign*(floor(abs(aangle)/(2*M_PI)))*2*M_PI;		// aangle modulo 2*Pi
			
	} else {
		angle = aangle;
	}
	
	radius = aradius;
	
	speed = aspeed;
}


void Camera::setAngle(double aangle)
{
	int sign;

	if (abs(aangle) > 2*M_PI)
	{
		sign = (aangle > 0) ? 1 : -1;									// signum von aangle
		angle = aangle - sign*(floor(abs(aangle)/(2*M_PI)))*2*M_PI;		// aangle modulo 2*Pi ("reelles modulo")
	} else {
		angle = aangle;
	}
}

void Camera::setSpeed(double aspeed)
{
	speed = aspeed;
}

void Camera::setRadius(double aradius)
{
	radius = aradius;
}

double Camera::getAngle(void)
{
	return angle;
}

double Camera::getHeight(void)
{
	return height;
}

/*
get the x axis
*/
double Camera::getFirstAxis(void)
{
	return radius*cos(angle);
}
double Camera::getFirstAxis(double angleOffset)
{
	return radius*cos(angle+angleOffset);
}
double Camera::getSecondAxis(void)
{
	return radius*sin(angle);
}
double Camera::getSecondAxis(double angleOffset)
{
	return radius*sin(angle+angleOffset);
}

double Camera::getSpeed(void)
{
	return speed;
}

double Camera::getRadius(void)
{
	return radius;
}

void Camera::doStep(void)
{
	angle += speed;

	if (angle >= 2*M_PI)
		angle -= 2*M_PI;
	if (angle < 0)
		angle += 2*M_PI;
}

double Camera::round(double x)
{
	if (x >= 0)
	{
		if (x - floor(x) > 0.5)
			return ceil(x);
		else
			return floor(x);
	} else {
		if (x - ceil(x) < -0.5)
			return floor(x);
		else
			return ceil(x);
	}
}