#include "environment.h"

// std libs
#include <stdio.h>
#include <math.h>

#define PI 3.14159265

#include "actordrawing.h"
#include "texturehelper.h"


// constants we use in the code
#define coneDepthBellowPlane 0.5
#define nailLength 0.2
#define nailRadius 0.01
#define boxRadius 0.6
#define pileRadius 0.2
#define capsuleRadius 0.2

NxReal Environment::radius = 3.0;
NxReal Environment::height = 2.0;

GLuint Environment::textureNails[1];

Environment::Environment(NxPhysicsSDK *gPhysicsSDK, NxScene *gScene)
{
	Environment::physicsSDK = gPhysicsSDK;
	Environment::scene = gScene;
	//Environment::radius = radius;
	//Environment::height = height;

	for(int i=0; i < 6; i++) {
		numbers[i] = *(new DigitalNumber(boxRadius/3.0f, boxRadius/1.5f));
	}

	TextureHelper::loadGLTextures(&textureNails[0], "football.bmp");

	createScoreBox();
	createSpotlights();
	createFloor();
	createPiles();
	createCapsules();
	createCone();
	createNails();
}
void Environment::createScoreBox() {
	NxBoxShapeDesc boxDesc;
	NxActorDesc box;
	boxDesc.dimensions = *(new NxVec3(boxRadius,boxRadius/2.0,boxRadius));
	box.shapes.pushBack(&boxDesc);
	box.globalPose.t = *(new NxVec3(0, height*1.45, 0));
	
	// da actor keinen body zugewiesen bekommen hat, gilt er als "kinematic"
	scene->createActor(box);
}
void Environment::drawScore(Player *players) {
	int values[] = { players[0].score / 100,
		            (players[0].score % 100) / 10 ,
					 players[0].score % 10 ,
		             players[1].score / 100,
					(players[1].score % 100) / 10 ,
		             players[1].score % 10  };


	glPushMatrix();
	//glTranslatef(0, height*1.3+boxRadius/2.0, 0);
	glTranslatef(0, height*1.45-(boxRadius/4.0), 0);
	for(int i=0; i<4; i++) {
		glPushMatrix();
		glTranslatef(0, 0, boxRadius+0.1);
		glTranslatef(-boxRadius, 0, 0);

		glNormal3f(0.0f, 0.0f, -1.0f);
		for(int i=0; i<6; i++) {
			if(i<3) {
				ActorDrawing::setMaterial(ActorDrawing::materialSun);
			} else {
				ActorDrawing::setMaterial(ActorDrawing::materialViolet);
			}
			numbers[i].draw(values[i]);
			glTranslatef(boxRadius/3.0f, 0, 0);
		}

		glPopMatrix();

		// draw energy
		glPushMatrix();
		glTranslatef(0.0f, -boxRadius/2.0f, boxRadius+0.1);

		ActorDrawing::setMaterial(ActorDrawing::materialSun);
		glRectf(-boxRadius, 0.0f, -boxRadius+(boxRadius*(players[0].energy/Player::maxenergy)), 0.1);

		ActorDrawing::setMaterial(ActorDrawing::materialViolet);
		glRectf(boxRadius-boxRadius*(players[1].energy/Player::maxenergy), 0.0f, boxRadius, 0.1);
		glPopMatrix();

		// rotate to next side
		glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
	}
	glPopMatrix();
	ActorDrawing::setMaterial(ActorDrawing::materialDefault);
}
void Environment::createSpotlights() {

	// i=-2 : -1 + (abs(i)%2)*2 = -1 , ((i*2+1)%2)*(-1) =  1
	// i=-1 : -1 + (abs(i)%2)*2 =  1 , ((i*2+1)%2)*(-1) =  1
	// i= 0 : -1 + (abs(i)%2)*2 = -1 , ((i*2+1)%2)*(-1) = -1
	// i= 1 : -1 + (abs(i)%2)*2 =  1 , ((i*2+1)%2)*(-1) = -1

	GLenum lightId[4] = {GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3};
	
	// default values
	float ambientColor[]	= { 1.0f, 1.0f, 1.0f, 1.0f };
	float diffuseColor[]	= { 1.0f, 1.0f, 1.0f, 1.0f };
	float specularColor[]	= { 1.0f, 1.0f, 1.0f, 1.0f };

	for(int i=-2; i<2; i++) {
		NxReal x = (-1 + (abs(i)%2)*2)*radius;
		NxReal y = 2.0*height;
		NxReal z = ((i*2+1)%2)*(-1)*radius;
		float pos[]	= {x, y, z, 1.0f };
		float dir[] = {0-x, -1.5*height, 0-z, 1.0f};

		glLightfv(lightId[i+2], GL_AMBIENT, ambientColor);
		glLightfv(lightId[i+2], GL_DIFFUSE, diffuseColor);
		glLightfv(lightId[i+2], GL_SPECULAR, specularColor);

		glLightfv(lightId[i+2], GL_POSITION, pos);
		glLightfv(lightId[i+2], GL_SPOT_DIRECTION, dir);

		glLightf(lightId[i+2], GL_SPOT_EXPONENT, 15.0f);
		glLightf(lightId[i+2], GL_SPOT_CUTOFF, 30.0f);

		glEnable(lightId[i+2]);
	}

}

/*
 * floor plane goes from Y: -0.2 to 0
 */
void Environment::createFloor() {

	// Create ground plane
	Plane *plane = new Plane();
	plane->normal.set(NxReal(0.0), NxReal(1.0), NxReal(0.0));
	plane->d = NxReal(0.0);
	TextureHelper::loadGLTextures(&plane->texture[0], "holz.bmp");

	NxPlaneShapeDesc planeDesc;
	planeDesc.normal.set(plane->normal);
	planeDesc.d = plane->d;
	planeDesc.userData = plane;
	planeDesc.shapeFlags |= NX_TRIGGER_ENABLE;

	NxActorDesc actorDesc;
	actorDesc.shapes.pushBack(&planeDesc);
	scene->createActor(actorDesc);
}

/*
 * the piles are located at every corner
 */
void Environment::createPiles() {
	NxBoxShapeDesc box;
	// i=-2 : -1 + (abs(i)%2)*2 = -1 , ((i*2+1)%2)*(-1) =  1
	// i=-1 : -1 + (abs(i)%2)*2 =  1 , ((i*2+1)%2)*(-1) =  1
	// i= 0 : -1 + (abs(i)%2)*2 = -1 , ((i*2+1)%2)*(-1) = -1
	// i= 1 : -1 + (abs(i)%2)*2 =  1 , ((i*2+1)%2)*(-1) = -1
	for(int i=-2; i<2; i++) {
		NxActorDesc pile;
		// bei der NxBox sind die angegebenen laengen die halblaengen der seiten
		box.dimensions = *(new NxVec3(pileRadius, height, pileRadius));
		pile.shapes.pushBack(&box);
		// y = height/2
		pile.globalPose.t = *(new NxVec3((-1 + (abs(i)%2)*2)*radius, height, ((i*2+1)%2)*(-1)*radius));
		scene->createActor(pile);
	}
}

/*
 * capsules are located at every middle of a plane side
 */
void Environment::createCapsules() {
	for(int i=-2; i<2; i++) {
		NxActorDesc capsule;

		GLuint texture[1];
		TextureHelper::loadGLTextures(&texture[0], "football.bmp");

		NxCapsuleShapeDesc capsuleShape;
		capsuleShape.height = height;
		capsuleShape.radius = capsuleRadius;
		capsuleShape.userData = &texture[0];

		capsule.shapes.pushBack(&capsuleShape);
		// i=-2 : (i%2) =  0 , ((i+1)%2) = -1
		// i=-1 : (i%2) = -1 , ((i+1)%2) =  0
		// i= 0 : (i%2) =  0 , ((i+1)%2) =  1
		// i= 1 : (i%2) =  1 , ((i+1)%2) =  0
		capsule.globalPose.t = *(new NxVec3((i%2)*radius, 0.0, ((i+1)%2)*radius));
		scene->createActor(capsule);
	}
}
void Environment::createConeTriangleFan() {
	glBegin(GL_TRIANGLE_FAN);
	glVertex3f(0, -1, 0);
	int subdivisions = 32;
	float angle = 0;
	float x = 0;
	float y = height;
	float z = 0;
	for(int i=0; i<subdivisions; i++) {
		x = radius * cos(angle);
		z = radius * sin(angle);
		glVertex3f(x, y, z);
		angle += 2*PI/subdivisions;
	}
	glEnd();
}
void Environment::createNails() {
	NxU32 subdivisions = 32;
	NxU32 slices = 10;
	NxReal angle = 2.0*PI;
	NxReal x = 0;
	NxReal y = 0;
	NxReal z = 0;
	for(NxU32 i=0; i<subdivisions; i++) {
		for(NxU32 j=2; j<slices; j++) {
			x = radius*((1.0f*j)/slices) * cos(angle);
			z = radius*((1.0f*j)/slices) * sin(angle);
			y = ((1.0f*j)/slices) * (height+coneDepthBellowPlane) - coneDepthBellowPlane;

			//glPushMatrix();
			//glTranslatef(x, y, z);

			NxCapsuleShapeDesc capsuleShape;
			capsuleShape.height = nailLength;
			capsuleShape.radius = nailRadius;
			capsuleShape.userData = &textureNails[0];

			NxActorDesc capsule;
			capsule.shapes.pushBack(&capsuleShape);
			capsule.globalPose.t = *(new NxVec3(x, y, z));
			scene->createActor(capsule);

			//glPopMatrix();
		}
		angle -= 2.0*PI/subdivisions;
	}
}
/*
 * the cone is defined by the center in the middle and points on a circle.
 * we use the triangle mesh from Novodex here
 */
void Environment::createCone() {
	// calculate vertices/normals on circle (this is an array with the number of subdivisions of vertices)
	NxU32 subdivisions = 32;
	NxVec3 *circleVertices = new NxVec3[subdivisions];
	NxVec3 *circleNormals = new NxVec3[subdivisions];
	NxReal angle = 2.0*PI;
	NxReal x = 0;
	NxReal y = height;
	NxReal z = 0;
	for(NxU32 i=0; i<subdivisions; i++) {
		x = radius * cos(angle);
		z = radius * sin(angle);
		NxVec3 & v = circleVertices[i];
		v.set(x, y, z);
		NxVec3 & n = circleNormals[i];
		n.set(0.0-x, y, 0.0-z);
		angle -= 2.0*PI/subdivisions;
	}

	cone = new TriangleMesh();

	cone->numVertices = 3*subdivisions;
	cone->numFaces = subdivisions;

	// used for easier handling
	NxU32 numVertices = cone->numVertices;
	NxU32 numFaces = cone->numFaces;

	// init cone vertices/normals (this must be 3 times the number of subdivisions for Novodex)
	cone->vertices   = new NxVec3[numVertices];
	cone->normals    = new NxVec3[numVertices];
	for(NxU32 i=0; i<numFaces; i++) {
		// vertices
		NxVec3 & v0       = cone->vertices[i*3 + 0];
		v0.set(0.0, -coneDepthBellowPlane, 0.0);
		NxVec3 & v1       = cone->vertices[i*3 + 1];
		v1.set(circleVertices[i]);
		NxVec3 & v2       = cone->vertices[i*3 + 2];
		v2.set(circleVertices[(i+1)%subdivisions]);
		// normals
		NxVec3 & n0       = cone->normals[i*3 + 0];
		n0.set(0.0, 1.0, 0.0);
		NxVec3 & n1       = cone->normals[i*3 + 1];
		n1.set(circleNormals[i]);
		NxVec3 & n2       = cone->normals[i*3 + 2];
		n2.set(circleNormals[(i+1)%subdivisions]);
		i = i;
	}

	// init cone faces, materials and textures
	cone->faces = new NxU32[numFaces*3];
	cone->materials = new NxMaterialIndex[numFaces];
	cone->textures = new GLuint[numFaces];
	GLuint texture[1];
	TextureHelper::loadGLTextures(&texture[0], "rock.bmp");
	//TextureHelper::loadGLTextures(&texture[1], "football.bmp");
	for(NxU32 i=0; i<numFaces; i++) {
		cone->faces[3*i]   = 3*i;
		cone->faces[3*i+1] = 3*i+1;
		cone->faces[3*i+2] = 3*i+2;
		cone->materials[i] = ActorDrawing::materialRock;
		cone->textures[i]  = texture[0];
	}

	// build physical model
	NxTriangleMeshDesc coneDesc;
	coneDesc.numVertices               = numVertices;
	coneDesc.numTriangles              = numFaces;
	coneDesc.pointStrideBytes          = sizeof(NxVec3);
	coneDesc.triangleStrideBytes       = 3*sizeof(NxU32);
	coneDesc.points                    = cone->vertices;
	coneDesc.triangles                 = cone->faces;
	coneDesc.flags                     = 0;
	coneDesc.materialIndexStride       = sizeof(NxMaterialIndex);
	coneDesc.materialIndices           = cone->materials;

	coneDesc.heightFieldVerticalAxis   = NX_Y;
	coneDesc.heightFieldVerticalExtent = -1000.0f;

	coneDesc.pmap                      = NULL;
	coneDesc.stream                    = NULL;

	NxTriangleMeshShapeDesc coneShapeDesc;
	coneShapeDesc.meshData             = physicsSDK->createTriangleMesh(coneDesc);
	coneShapeDesc.userData             = cone;

	// create the actor and its shape
	NxActorDesc actorDesc;
	actorDesc.shapes.pushBack(&coneShapeDesc);
	actorDesc.globalPose.t = *(new NxVec3(0, 0, 0));
	NxActor *actor = scene->createActor(actorDesc);
	//NxShape *shape = actor->createShape(coneShapeDesc);
	//shape->userData = coneNormals;
}



Environment::~Environment(void)
{
}
