#define NOMINMAX

#include <windows.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <GL/glu.h>							// Header File For The GLu32 Library
#include <GL/glaux.h>							// Header File For The GLaux Library

#include <stdio.h>
#include <time.h>

#define _USE_MATH_DEFINES	// in math.h werden sonst pi etc. nicht defniert
#include <math.h>

// Physics code
#undef random
#include "NxPhysics.h"
#include "ErrorStream.h"

#include "trianglemesh.h"
#include "plane.h"
#include "texturehelper.h"

#include "actordrawing.h"
#include "environment.h"
#include "camera.h"
#include "globe.h"
#include "triggercallback.h"
#include "player.h"

//static UserAllocator*	gMyAllocator = NULL;
static bool				gPause = false;
static NxPhysicsSDK*	gPhysicsSDK = NULL;
static NxScene*			gScene = NULL;
static NxVec3			gDefaultGravity(0.0f, -9.81f, 0.0f);
static ErrorStream		gErrorStream;

static int gMainHandle;
static bool gShadows = true;
static float gRatio = 1.0f;

Environment *environment;
Camera *camera;
Player players[2];
NxActorDesc * newNails[10];
NxActor * shootsToRemove[10];
int doLighting = 1;

#define nailLength 0.2
#define nailRadius 0.01

#define aimDelta 0.1
#define marbleInjectProb 0.05

#define energyAdd 0.05f

static int mx = 0;
static int my = 0;

// forward decl
void shoot(Player player);

static void InitNx()
{
	//gMyAllocator = new UserAllocator;

	// Initialize PhysicsSDK
	gPhysicsSDK = NxCreatePhysicsSDK(NX_PHYSICS_SDK_VERSION, 0, &gErrorStream);
	if(!gPhysicsSDK)	return;

	gPhysicsSDK->setParameter(NX_MIN_SEPARATION_FOR_PENALTY, -0.005f);

	NxMaterial	defaultMaterial;
	defaultMaterial.restitution		= 0.2f; //mechanik: stosszahl. 1.0 fuer 100 prozent impulserhaltung
	defaultMaterial.staticFriction	= 0.5f;
	defaultMaterial.dynamicFriction	= 0.5f;
	gPhysicsSDK->setMaterialAtIndex(0, &defaultMaterial);
	ActorDrawing::materialDefault = 0;

	NxMaterial	m;
	//terrain materials:

	m.restitution		= 1.0f;
	m.staticFriction	= 0.0f;
	m.dynamicFriction	= 0.0f;
	ActorDrawing::materialIce = gPhysicsSDK->addMaterial(m);
	m.restitution		= 0.3f;
	m.staticFriction	= 1.2f;
	m.dynamicFriction	= 1.0f;
	ActorDrawing::materialRock = gPhysicsSDK->addMaterial(m);
	m.restitution		= 0.0f;
	m.staticFriction	= 3.0f;
	m.dynamicFriction	= 1.0f;
	ActorDrawing::materialMud = gPhysicsSDK->addMaterial(m);
	m.restitution		= 0.0f;
	m.staticFriction	= 0.0f;
	m.dynamicFriction	= 0.0f;
	ActorDrawing::materialGrass = gPhysicsSDK->addMaterial(m);
	m.restitution		= 0.0f;
	m.staticFriction	= 0.0f;
	m.dynamicFriction	= 0.0f;
	ActorDrawing::materialSun = gPhysicsSDK->addMaterial(m);
	m.restitution		= 0.0f;
	m.staticFriction	= 0.0f;
	m.dynamicFriction	= 0.0f;
	ActorDrawing::materialViolet = gPhysicsSDK->addMaterial(m);



	// Create a scene
	NxSceneDesc sceneDesc;
	sceneDesc.gravity				= gDefaultGravity;
	sceneDesc.broadPhase			= NX_BROADPHASE_COHERENT;
	sceneDesc.collisionDetection	= true;
	gScene = gPhysicsSDK->createScene(sceneDesc);

}

void RenderCallback()
{
	float glmat[16];

	// inject marbles
	if( (1.0*rand()/RAND_MAX) < marbleInjectProb ) {
		Globe::createGlobe(gScene, players);
	}

	gScene->simulate(1.0f/60.0f);

	if(players[0].energy < Player::maxenergy - energyAdd) {
		players[0].energy += energyAdd;
	}
	if(players[1].energy < Player::maxenergy - energyAdd) {
		players[1].energy += energyAdd;
	}
	
	// Clear buffers
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Setup camera
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0f, gRatio, 1, 10000);

	camera->doStep();
	double r = camera->getRadius()*1.4;
	double angle = camera->getAngle();
	gluLookAt(
		r*cos(angle)          , camera->getHeight()*1.45 , r*sin(angle), 
		0                     , Environment::height/2.0 , 0                      ,
		0                     , 1                       , 0); // eye coords, point in scene to look at, up vector

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// Keep physics & graphics in sync (by displaying graphics)
	int nbActors = gScene->getNbActors();
	NxActor** actors = gScene->getActors();
	while(nbActors--)
	{
		// get next actor
		NxActor* actor = *actors++;

		// get shape and draw it
		NxShape * shape = actor->getShapes()[0];

		glPushMatrix();
		shape->getGlobalPose().getColumnMajor44(glmat);
		glMultMatrixf(glmat);

		switch(shape->getType())
		{
		case NX_SHAPE_PLANE:
			ActorDrawing::drawPlane(shape);
			break;

		case NX_SHAPE_SPHERE:
			ActorDrawing::drawSphere(shape);
			break;

		case NX_SHAPE_BOX:
			ActorDrawing::drawBox(shape);
			break;

		case NX_SHAPE_CAPSULE:
			ActorDrawing::drawCapsule(shape);
			break;

		case NX_SHAPE_MESH:
			ActorDrawing::drawMesh(shape);
			break;
		}
		glPopMatrix();

	}

	// draw score
	environment->drawScore(players);

	glFlush();
	glutSwapBuffers();
	
	gScene->flushStream();
    gScene->fetchResults(NX_RIGID_BODY_FINISHED, true);

	// remove shoots and create new nail
	for(int i=0; i<10; i++) {
		if(shootsToRemove[i] != 0) {
			gScene->releaseActor(*shootsToRemove[i]);
			shootsToRemove[i] = 0;
		}
	}
	for(int i=0; i<10; i++) {
		if(newNails[i] != 0) {
			gScene->createActor(*newNails[i]);
			newNails[i] = 0;
		}
	}


	// release unused actors
	nbActors = gScene->getNbActors();
	actors = gScene->getActors();
	while(nbActors--) {
		NxActor* actor = *actors++;

		// calculate its y pos
		NxReal y_pos = actor->getGlobalPose().t.y;
		if(y_pos < 0) {
			gScene->releaseActor(*actor);
		}
	}
}

void ReshapeCallback(int width, int height)
{
	glViewport(0, 0, width, height);
	gRatio = float(width)/float(height);
}

void IdleCallback()
{
	glutPostRedisplay();
}


void KeyboardCallback(unsigned char key, int mouse_x, int mouse_y) {

	switch(key)
	{
	case 27	: exit(0);break; // 27 = ESC
	case 'w': 
	case 'W': players[0].aimy += aimDelta; break;

	case 'S':
	case 's': players[0].aimy -= aimDelta; break;

	case 'A':
	case 'a': players[0].aimx -= aimDelta; break;

	case 'D':
	case 'd': players[0].aimx += aimDelta; break;

	case '\t':
		if(players[0].energy > 1.0f) {
			shoot(players[0]);
			players[0].energy -= 1.0f;
		}
		break;
	case 13  :// 13 = ENTER
		if(players[1].energy > 1.0f) {
			shoot(players[1]);
			players[1].energy -= 1.0f;
		}
		break; 

	case 'l':
		if(doLighting == 1) {
			glDisable(GL_LIGHTING);
			doLighting = 0;
		} else {
			glEnable(GL_LIGHTING);
			doLighting = 1;
		}
	}
	printf("aim of player 0: angle=%f, y=%f\n",players[0].aimx, players[0].aimy);

}
void Keyboard2Callback(int key, int mouse_x, int mouse_y) {
	switch(key) {
	case GLUT_KEY_UP:    players[1].aimy += aimDelta; break;
	case GLUT_KEY_DOWN:  players[1].aimy -= aimDelta; break;
	case GLUT_KEY_LEFT:  players[1].aimx -= aimDelta; break;
	case GLUT_KEY_RIGHT: players[1].aimx += aimDelta; break;
	}
	printf("aim of player 1: angle=%f, y=%f\n",players[1].aimx, players[1].aimy);
}
static void MouseCallback(int button, int state, int x, int y)
{
	mx = x;
	my = y;
}
static void MotionCallback(int x, int y)
{
	players[0].aimx -= (mx-x)/500.0;
	players[0].aimy += (my-y)/500.0;
	mx = x;
	my = y;
}
int main(int argc, char ** argv)
{
	glutInit(&argc, argv);
	glutInitWindowSize(512, 512);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	gMainHandle = glutCreateWindow("NxTest");
	glutSetWindow(gMainHandle);
	glutDisplayFunc(RenderCallback);
	glutReshapeFunc(ReshapeCallback);
	glutIdleFunc(IdleCallback);
	glutKeyboardFunc(KeyboardCallback);
	glutSpecialFunc(Keyboard2Callback);
	glutMouseFunc(MouseCallback);
	glutMotionFunc(MotionCallback);

	//glutExitFunc(ExitCallback);

	// Setup default render states
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	//glEnable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glShadeModel(GL_SMOOTH);

	// setup random numbers
	srand((unsigned int) time(NULL));

	// Physics code
	InitNx();
	// ~Physics code

	environment = new Environment(gPhysicsSDK, gScene);

	camera = new Camera();

	Globe::initialize(camera);

	for(int i=0; i<10; i++) {
		shootsToRemove[i] = 0;
		newNails[i] = 0;
	}

	TriggerCallback *tc = new TriggerCallback();
	tc->setScene(gScene);
	gScene->setUserTriggerReport(tc);

	GLuint texture1 = -1;
	GLuint texture2 = -1;
	TextureHelper::loadGLTextures(&texture1, "player1.bmp");
	TextureHelper::loadGLTextures(&texture2, "player2.bmp");
	players[0] = *(new Player(0, texture1,  8.0/180.0*M_PI));
	players[1] = *(new Player(1, texture2, -8.0/180.0*M_PI));

	printf("left  player: w,s,a and d (or the mouse) to aim and tab to fire.\n");
	printf("right player: arrows to aim and enter to fire.\n");

	// Run
	glutMainLoop();
	

	// release
	if(gPhysicsSDK && gScene)	gPhysicsSDK->releaseScene(*gScene);
	gPhysicsSDK->release();
}

void shoot(Player player) {

	/*NxCapsuleShapeDesc capsuleDesc;
	capsuleDesc.height = nailLength*1.5;
	capsuleDesc.radius = nailRadius*3;
	capsuleDesc.userData = &player.texture[0];

	NxActorDesc actor;
	actor.density = 20;
	actor.shapes.pushBack(&capsuleDesc);

	NxCapsuleShapeDesc capsuleTriggerDesc = capsuleDesc;
	capsuleTriggerDesc.radius = capsuleTriggerDesc.radius*1.1;
	capsuleTriggerDesc.flags |= NX_TRIGGER_ENABLE*/;
	
	NxActorDesc actor;
	//NxBodyDesc body;
	NxBoxShapeDesc nail;
	nail.dimensions = NxVec3(0.05, 0.2, 0.05);
	actor.shapes.pushBack(&nail);

	NxBoxShapeDesc nailTrigger;
	nailTrigger.dimensions = NxVec3(0.05, 0.2, 0.05);
	nail.shapeFlags |= NX_TRIGGER_ENABLE;
	actor.shapes.pushBack(&nailTrigger);
	
	actor.density = 20;

	// ausrichtung berechnen
	double angle = camera->getAngle()+player.aimx;
	double camr = camera->getRadius();

	NxVec3 direction = NxVec3(0-camr*cos(angle),
		                      player.aimy-camera->getHeight(),
							  0-camr*sin(angle));
	direction.normalize();

	// matrix for rotation around y
	double angle_y = -1 * atan2((double) direction.z, (double)direction.x);
	NxMat33 rot_y = NxMat33(NxQuat(angle_y/M_PI*180.0, NxVec3(0,1,0)));

	// matrix for slant of nail
	double r = sqrt(direction.x*direction.x + direction.z*direction.z);
	double angle_z = atan2((double)direction.y, r);

	NxMat33 rot_z = NxMat33(NxQuat(angle_z/M_PI*180.0+90.0, NxVec3(0,0,1)));

	NxMat33 rot = rot_y*rot_z;
	
	NxVec3 pos = NxVec3(camera->getFirstAxis(player.posAngleOffset),
		                camera->getHeight()+0.1,
						camera->getSecondAxis(player.posAngleOffset));
	actor.globalPose = NxMat34 (rot,pos);
	
	direction.normalize();

	NxBodyDesc body;
	body.linearVelocity = direction * 10.0;
	actor.body = &body;
	gScene->createActor(actor);
}
